# Generated by roxygen2: do not edit by hand

export(bdapply)
export(bdmerge)
export(closebd5con)
export(genapply)
export(gentobd)
export(getaaf)
export(getbd5snp)
export(getbd5snp_buf)
export(getbd5snp_con)
export(getbdinfo)
export(getgeninfo)
export(getmaf)
export(getrsq)
export(getsnp)
export(getvcfinfo)
export(mergebd)
export(openbd5con)
export(subsetbd)
export(updatebd)
export(vcfapply)
export(vcftobd)
export(vcftobdlegacy)
importFrom(Rcpp,sourceCpp)
importFrom(digest,digest)
importFrom(prodlim,row.match)
importFrom(stats,var)
importFrom(utils,read.table)
useDynLib(BinaryDosage, .registration = TRUE)
