% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert.R
\name{bd1tobd5}
\alias{bd1tobd5}
\title{Convert a format 1 binary dosage file to Format 5}
\usage{
bd1tobd5(bdfiles, bdose_file)
}
\arguments{
\item{bdfiles}{Vector of three file names for the format 1 binary dosage
file: the binary dosage file, the family file, and the map file.}

\item{bdose_file}{Path for the output .bdose file. The companion .bdi
metadata file is written to \code{paste0(bdose_file, ".bdi")}.}
}
\value{
NULL (invisibly)
}
\description{
Reads a format 1 binary dosage file (subformat 1 or 2) and converts it to
a Format 5 file pair. If the source file does not contain genotype
probabilities (subformat 1), those values are stored as missing in the
output.
}
\keyword{internal}
